<?php
/**
Plugin Name: Delicious XML Importer v0.1

Description: Will process Delicious' XML export for importing posts into WordPress. Heavily based on the RSS Importer included in WordPress. Cleaned up & made workable with WordPress 3 by Pat Ramsey.

Author: Guillermo Moreno Quiroga 
Author URI: http://memografia.com/wordpress/delicious-to-wordpress-importer/
Version: 0.1a
Stable tag: 0.1a
License: GPL v2 - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

if ( !defined('WP_LOAD_IMPORTERS') )
	return;

// Load Importer API
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( !class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) )
		require_once $class_wp_importer;
}

if ( class_exists( 'WP_Importer' ) ) {
class Delicious_Import extends WP_Importer {

	var $posts = array ();
	var $file;

	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>'.__('Import Delicious XML').'</h2>';
	}

	function footer() {
		echo '</div>';
	}

	function unhtmlentities($string) { // From php.net for < 4.3 compat
		$trans_tbl = get_html_translation_table(HTML_ENTITIES);
		$trans_tbl = array_flip($trans_tbl);
		return strtr($string, $trans_tbl);
	}

	function greet() {
		echo '<div class="narrow">';
		echo '<p>'.__('This importer allows you to extract posts from a Delicious XML export file into your blog. Heavily based on the RSS Importer included in WordPress.').'</p>';
		echo '<form enctype="multipart/form-data" method="post" action="admin.php?import=delicious&amp;step=1"><p>';
		wp_nonce_field('delicious-import');
		echo '<label for="upload">Choose a file from your computer:</label>';
		echo '<input type="file" id="upload" name="import" size="25" />';
		echo '<input type="hidden" name="action" value="save" />';
		echo '<p><label for="categoriestags">Do you want your Delicious Tags as </label>';
		echo ' <select name="categoriestags"><option>Categories</option><option>Tags</option></select>?</p>';
		echo '</p>';
		echo '<p class="submit">';
		echo '<input type="submit" name="submit" class="button" value="'.esc_attr__('Submit').'" />';
		echo '</p>';
		echo '</form>';
		echo '</div>';
	}

	function get_posts() {
		global $wpdb;
		set_magic_quotes_runtime(0);
		$datalines = file($this->file); // Read the file into an array
		$importdata = implode('', $datalines); // squish it
		$importdata = str_replace(array ("\r\n", "\r"), "\n", $importdata);
		$xml = simplexml_load_string($importdata);
		$index = 0;
		foreach($xml->post as $post) {
		$post_title = $post['description'];
		$post_date_gmt = $post['time'];
		$post_date_gmt = strtotime($post_date_gmt);
		$post_date_gmt = gmdate('Y-m-d H:i:s', $post_date_gmt);
		$post_date = get_date_from_gmt( $post_date_gmt );
		$category = $post['tag'];
		$categories = explode(" ", $category);
		$cat_index = 0;
		foreach ($categories as $category) {
			$categories[$cat_index] = $wpdb->escape($this->unhtmlentities($category));
			$cat_index++;
		}
		$post_content = $post['extended'];
		$post_content = $wpdb->escape($this->unhtmlentities(trim($post_content)));

		$post_link = $post['href'];
		$post_link = $wpdb->escape($this->unhtmlentities(trim($post_link)));
		$post_link = '<p class="delicious_post_link"><a href="'.$post_link.'">'.$post_title.'</a></p>';
		$post_content = $post_content.$post_link;
		
			// Clean up content
			$post_content = preg_replace_callback('|<(/?[A-Z]+)|', create_function('$match', 'return "<" . strtolower($match[1]);'), $post_content);
			$post_content = str_replace('<br>', '<br />', $post_content);
			$post_content = str_replace('<hr>', '<hr />', $post_content);

			$post_author = 1;
			$post_status = 'publish';
			$this->posts[$index] = compact('post_author', 'post_date', 'post_date_gmt', 'post_content', 'post_title', 'post_status', 'categories');
			$index++;
		}
	}

	function import_posts() {
		echo '<ol>';
		$categoriestags = $_POST['categoriestags'];
		$cat_id = $_POST['cat_id'];
		foreach ($this->posts as $post) {
			echo "<li>".__('Importing post...');

			extract($post);

			if ($post_id = post_exists($post_title, $post_content, $post_date)) {
				_e('Post already imported');
			} else {
				$post_id = wp_insert_post($post);
				if ( is_wp_error( $post_id ) )
					return $post_id;
				if (!$post_id) {
					_e('Couldn&#8217;t get post ID');
					return;
				}
				if (0 != count($categories)) 
					if ($categoriestags == 'Tags') {
					wp_add_post_tags($post_id, $categories);
					}
					else {
					wp_create_categories($categories, $post_id);
					} 
				_e('Done !');
			}
			echo '</li>';
		}

		echo '</ol>';

	}

	function import() {
		$file = wp_import_handle_upload();
		if ( isset($file['error']) ) {
			echo $file['error'];
			return;
		}

		$this->file = $file['file'];
		$this->get_posts();
		$result = $this->import_posts();
		if ( is_wp_error( $result ) )
			return $result;
		wp_import_cleanup($file['id']);
		do_action('import_done', 'delicious');

		echo '<h3>';
		printf(__('All done. <a href="%s">Have fun!</a>'), get_option('home'));
		echo '</h3>';
	}

	function dispatch() {
		if (empty ($_GET['step']))
			$step = 0;
		else
			$step = (int) $_GET['step'];

		$this->header();

		switch ($step) {
			case 0 :
				$this->greet();
				break;
			case 1 :
				check_admin_referer('delicious-import');
				$result = $this->import();
				if ( is_wp_error( $result ) )
					echo $result->get_error_message();
				break;
		}

		$this->footer();
	}

	function Delicious_Import() {
		// Nothing.
	}
}

$delicious_import = new Delicious_Import();

register_importer('delicious', __('Delicious'), __('Import posts from a Delicious XML export file.'), array ($delicious_import, 'dispatch'));

}
function delicious_importer_init() {
    load_plugin_textdomain( 'delicious-importer', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}
add_action( 'init', 'delicious_importer_init' );

?>